// ====================================================================
// This file is part of FlexibleSUSY.
//
// FlexibleSUSY is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// FlexibleSUSY is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with FlexibleSUSY.  If not, see
// <http://www.gnu.org/licenses/>.
// ====================================================================

// File generated at Tue 22 Jan 2019 17:31:20

/**
 * @file NUTSMSSM_two_scale_model.hpp
 * @brief contains class for model with routines needed to solve boundary
 *        value problem using the two_scale solver by solving EWSB
 *        and determine the pole masses and mixings
 *
 * This file was generated at Tue 22 Jan 2019 17:31:20 with FlexibleSUSY
 * 2.3.0 (git commit: b5dda61ad35a8ffff74bde70f63e1c2b815e751a) and SARAH 4.14.1 .
 */

#ifndef NUTSMSSM_TWO_SCALE_H
#define NUTSMSSM_TWO_SCALE_H

#include "NUTSMSSM_model.hpp"
#include "NUTSMSSM_mass_eigenstates.hpp"

#include "model.hpp"

namespace flexiblesusy {

class Two_scale;
/**
 * @class NUTSMSSM<Two_scale>
 * @brief model class with routines for determining masses and mixings and EWSB
 */
template<>
class NUTSMSSM<Two_scale> : public Model, public NUTSMSSM_mass_eigenstates {
public:
   explicit NUTSMSSM(const NUTSMSSM_input_parameters& input_ = NUTSMSSM_input_parameters());
   NUTSMSSM(const NUTSMSSM&) = default;
   NUTSMSSM(NUTSMSSM&&) = default;
   virtual ~NUTSMSSM() = default;
   NUTSMSSM& operator=(const NUTSMSSM&) = default;
   NUTSMSSM& operator=(NUTSMSSM&&) = default;

   // interface functions
   virtual void calculate_spectrum() override;
   virtual void clear_problems() override;
   virtual std::string name() const override;
   virtual void run_to(double scale, double eps = -1.0) override;
   virtual void print(std::ostream& out = std::cerr) const override;
   virtual void set_precision(double) override;
};

std::ostream& operator<<(std::ostream&, const NUTSMSSM<Two_scale>&);

} // namespace flexiblesusy

#endif
